import '../provider/data.dart';
import '../utils/board_utils.dart';
import '../utils/game_utils.dart';

class GameUtils {

  static Future<void> resetGame(Data myProvider) async {
    myProvider.updateStateRunning = false;
  }

  static Future<void> startGame(Data myProvider) async {
    myProvider.updateSize = myProvider.size;
    myProvider.updateStateRunning = true;
    myProvider.updateCells = BoardUtils.createEmptyBoard(myProvider.blockSizeHorizontal * myProvider.blockSizeVertical);
    BoardUtils.pickGrid(myProvider);
  }

  static void showTip(Data myProvider) {
    if (myProvider.currentCellCol == null || myProvider.currentCellRow == null) {
      // no selected cell -> pick one
      GameUtils.helpSelectCell(myProvider);
    } else {
      // currently selected cell -> set value
      GameUtils.helpFillCell(myProvider);
    }
  }

  static void helpSelectCell(Data myProvider) {
    List cells = myProvider.cells;
    int boardSize = myProvider.blockSizeHorizontal * myProvider.blockSizeVertical;

    // pick one of conflicting cells, if found
    List conflictingCells = [];
    for (var row = 0; row < boardSize; row++) {
      for (var col = 0; col < boardSize; col++) {
        if (!cells[row][col].isFixed && cells[row][col].value != 0) {

          if (cells[row][col].conflictsCount != 0 && !BoardUtils.isValueAllowed(myProvider, col, row, cells[row][col].value)) {
            conflictingCells.add([col, row]);
          }
        }
      }
    }
    if (conflictingCells.length != 0) {
      GameUtils.pickRandomFromList(myProvider, conflictingCells);
      return;
    }

    //  pick one form cells with unique non-conflicting candidate value
    List candidateCells = [];
    for (var row = 0; row < boardSize; row++) {
      for (var col = 0; col < boardSize; col++) {
        if (cells[row][col].value == 0) {
          int allowedValuesCount = 0;
          for (var value = 1; value <= boardSize; value++) {
            if (BoardUtils.isValueAllowed(myProvider, col, row, value)) {
              allowedValuesCount++;
            }
          }
          if (allowedValuesCount == 1) {
            candidateCells.add([col, row]);
          }
        }
      }
    }
    if (candidateCells.length != 0) {
      GameUtils.pickRandomFromList(myProvider, candidateCells);
      return;
    }
  }

  static void pickRandomFromList(Data myProvider, List cellsCoordinates) {
    if (cellsCoordinates.length > 0) {
      cellsCoordinates.shuffle();
      List cell = cellsCoordinates[0];
      myProvider.selectCell(cell[0], cell[1]);
    }
  }

  static void helpFillCell(Data myProvider) {
    int boardSize = myProvider.blockSizeHorizontal * myProvider.blockSizeVertical;

    int eligibleValue = 0;

    // ensure there is only one eligible value for this cell
    int allowedValuesCount = 0;
    for (var value = 1; value <= boardSize; value++) {
      if (BoardUtils.isValueAllowed(myProvider, myProvider.currentCellCol, myProvider.currentCellRow, value)) {
        allowedValuesCount++;
        eligibleValue = value;
      }
    }

    myProvider.updateCellValue(myProvider.currentCellCol, myProvider.currentCellRow, allowedValuesCount == 1 ? eligibleValue : 0);
    myProvider.selectCell(null, null);
    BoardUtils.computeConflictsInBoard(myProvider);
  }
}
